@@ -30,6 +30,7 @@
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QTimer>
+#include <QtCore/QCommandLineParser>
 #include <QApplication>
 
 #include <QDBusConnection>
@@ -46,16 +47,20 @@
 #include <KPluginInfo>
 #include <KPluginMetaData>
 
-Q_DECLARE_LOGGING_CATEGORY(KDED);
-Q_LOGGING_CATEGORY(KDED, "kf5.kded");
+Q_DECLARE_LOGGING_CATEGORY(KDED)
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
+// logging category for this framework, default: log stuff >= warning
+Q_LOGGING_CATEGORY(KDED, "kf5.kded", QtWarningMsg)
+#else
+Q_LOGGING_CATEGORY(KDED, "kf5.kded")
+#endif
 
 Kded *Kded::_self = 0;
 
-static bool checkStamps;
 static bool delayedCheck;
 static bool bCheckSycoca;
 static bool bCheckUpdates;
-static bool s_checkStampsDefault = true;
 
 #ifdef Q_DBUS_EXPORT
 extern Q_DBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
@@ -63,33 +68,6 @@ extern Q_DBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
 extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
 #endif
 
-static void runBuildSycoca(QObject *callBackObj = 0, const char *callBackSlot = 0)
-{
-    const QString exe = QStandardPaths::findExecutable(KBUILDSYCOCA_EXENAME);
-    Q_ASSERT(!exe.isEmpty());
-    QStringList args;
-    if (QStandardPaths::isTestModeEnabled()) {
-        args.append("--testmode");
-    }
-    if (checkStamps) {
-        args.append("--checkstamps");
-    }
-    if (delayedCheck) {
-        args.append("--nocheckfiles");
-    } else {
-        checkStamps = false;    // useful only during kded startup
-    }
-    if (callBackObj) {
-        QVariantList argList;
-        argList << exe << args << QStringList() << QString();
-        KToolInvocation::ensureKdeinitRunning();
-        QDBusInterface klauncher(QStringLiteral("org.kde.klauncher5"), QStringLiteral("/KLauncher"), QStringLiteral("org.kde.KLauncher"), QDBusConnection::sessionBus());
-        klauncher.callWithCallback("kdeinit_exec_wait", argList, callBackObj, callBackSlot);
-    } else {
-        KToolInvocation::kdeinitExecWait(exe, args);
-    }
-}
-
 static void runKonfUpdate()
 {
     KToolInvocation::kdeinitExecWait(QStringLiteral(KCONF_UPDATE_EXE),
@@ -99,8 +77,6 @@ static void runKonfUpdate()
 Kded::Kded()
     : m_pDirWatch(0)
     , m_pTimer(new QTimer(this))
-    , m_recreateCount(0)
-    , m_recreateBusy(false)
     , m_needDelayedCheck(false)
 {
     _self = this;
@@ -164,11 +140,14 @@ void Kded::messageFilter(const QDBusMessage &message)
         return;
     }
 
-    KDEDModule *module = self()->loadModule(obj, true);
-    if (!module) {
-        qCWarning(KDED) << "Failed to load module for " << obj;
-    }
-    Q_UNUSED(module);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+    // messageFilter runs in a secondary thread since Qt 5.6, so we use invokeMethod
+    // to load the module in the main thread. But we need to block so that the object
+    // can then process the message.
+    QMetaObject::invokeMethod(self(), "loadModule", Qt::BlockingQueuedConnection, Q_ARG(QString, obj), Q_ARG(bool, true));
+#else
+    self()->loadModule(obj, true);
+#endif
 }
 
 static int phaseForModule(const KPluginMetaData &module)
@@ -443,9 +422,6 @@ QStringList Kded::loadedModules()
 void Kded::slotKDEDModuleRemoved(KDEDModule *module)
 {
     m_modules.remove(module->moduleName());
-    //KLibrary *lib = m_libs.take(module->moduleName());
-    //if (lib)
-    //   lib->unload();
 }
 
 void Kded::slotApplicationRemoved(const QString &name)
@@ -531,19 +507,19 @@ void Kded::runDelayedCheck()
 
 void Kded::recreate(bool initial)
 {
-    m_recreateBusy = true;
     // Using KLauncher here is difficult since we might not have a
     // database
 
     if (!initial) {
         updateDirWatch(); // Update tree first, to be sure to miss nothing.
-        runBuildSycoca(this, SLOT(recreateDone()));
+        KSycoca::self()->ensureCacheValid();
+        recreateDone();
     } else {
         if (!delayedCheck) {
             updateDirWatch();    // this would search all the directories
         }
         if (bCheckSycoca) {
-            runBuildSycoca();
+            KSycoca::self()->ensureCacheValid();
         }
         recreateDone();
         if (delayedCheck) {
@@ -561,19 +537,7 @@ void Kded::recreateDone()
 {
     updateResourceList();
 
-    for (; m_recreateCount; m_recreateCount--) {
-        QDBusMessage msg = m_recreateRequests.takeFirst();
-        QDBusConnection::sessionBus().send(msg.createReply());
-    }
-    m_recreateBusy = false;
-
-    // Did a new request come in while building?
-    if (!m_recreateRequests.isEmpty()) {
-        m_pTimer->start(2000);
-        m_recreateCount = m_recreateRequests.count();
-    } else {
-        initModules();
-    }
+    initModules();
 }
 
 void Kded::dirDeleted(const QString &path)
@@ -583,23 +547,7 @@ void Kded::dirDeleted(const QString &path)
 
 void Kded::update(const QString &)
 {
-    if (!m_recreateBusy) {
-        m_pTimer->start(10000);
-    }
-}
-
-void Kded::recreate(const QDBusMessage &msg)
-{
-    if (!m_recreateBusy) {
-        if (m_recreateRequests.isEmpty()) {
-            m_pTimer->start(0);
-            m_recreateCount = 0;
-        }
-        m_recreateCount++;
-    }
-    msg.setDelayedReply(true);
-    m_recreateRequests.append(msg);
-    return;
+    m_pTimer->start(10000);
 }
 
 void Kded::readDirectory(const QString &_path)
@@ -615,49 +563,8 @@ void Kded::readDirectory(const QString &_path)
 
     Q_ASSERT(path != QDir::homePath());
     m_pDirWatch->addDir(path, KDirWatch::WatchFiles | KDirWatch::WatchSubDirs);       // add watch on this dir
-    return; // KDirWatch now claims to also support recursive watching
-#if 0
-    QDir d(_path, QString(), QDir::Unsorted, QDir::Readable | QDir::Executable | QDir::Dirs | QDir::Hidden);
-    // set QDir ...
-
-    //************************************************************************
-    //                           Setting dirs
-    //************************************************************************
-
-    if (!d.exists()) {                            // exists&isdir?
-        qCDebug(KDED) << "Does not exist:" << _path;
-        return;                             // return false
-    }
-
-    // Note: If some directory is gone, dirwatch will delete it from the list.
-
-    //************************************************************************
-    //                               Reading
-    //************************************************************************
-    QString file;
-    unsigned int i;                           // counter and string length.
-    unsigned int count = d.count();
-    for (i = 0; i < count; i++) {                       // check all entries
-        if (d[i] == "." || d[i] == ".." || d[i] == "magic") {
-            continue;    // discard those ".", "..", "magic"...
-        }
-
-        file = path;                           // set full path
-        file += d[i];                          // and add the file name.
-
-        readDirectory(file);        // yes, dive into it.
-    }
-#endif
 }
 
-/*
-bool Kded::isWindowRegistered(long windowId) const
-{
-  return m_globalWindowIdList.contains(windowId);
-
-}
-*/
-
 void Kded::registerWindowId(qlonglong windowId, const QString &sender)
 {
     if (!m_windowIdList.contains(sender)) {
@@ -748,25 +655,29 @@ KBuildsycocaAdaptor::KBuildsycocaAdaptor(QObject *parent)
 {
 }
 
-void KBuildsycocaAdaptor::recreate(const QDBusMessage &msg)
+void KBuildsycocaAdaptor::recreate()
 {
-    Kded::self()->recreate(msg);
+    Kded::self()->recreate();
 }
 
+// KF6: remove
 bool KBuildsycocaAdaptor::isTestModeEnabled()
 {
     return QStandardPaths::isTestModeEnabled();
 }
 
+// KF6: remove
 void KBuildsycocaAdaptor::enableTestMode()
 {
     QStandardPaths::enableTestMode(true);
 }
 
-static void setupAppInfo(QCoreApplication *app)
+static void setupAppInfo(QApplication *app)
 {
     app->setApplicationName("kded5");
+    app->setApplicationDisplayName("KDE Daemon");
     app->setOrganizationDomain("kde.org");
+    app->setApplicationVersion(KDED_VERSION_STRING);
 }
 
 extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
@@ -774,28 +685,30 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
     //options.add("check", qi18n("Check Sycoca database only once"));
 
     // WABA: Make sure not to enable session management.
-    putenv(qstrdup("SESSION_MANAGER="));
+    qunsetenv("SESSION_MANAGER");
+
+    // In older versions, QApplication creation was postponed until after
+    // testing for --check, in which case, only a QCoreApplication was created.
+    // Since that option is no longer used at startup, we removed that speed
+    // optimization for code clarity and easier support of standard parameters.
+    QApplication app(argc, argv);
+    setupAppInfo(&app);
+    app.setQuitOnLastWindowClosed(false);
+
+    QCommandLineParser parser;
+    parser.addHelpOption();
+    parser.addVersionOption();
+    parser.addOption(QCommandLineOption(QStringLiteral("check"), QStringLiteral("Check cache validity")));
+    parser.process(app);
 
     // Parse command line before checking D-Bus
-    if (argc > 1 && QByteArray(argv[1]) == "--check") {
+    if (parser.isSet(QStringLiteral("check"))) {
         // KDBusService not wanted here.
-        QCoreApplication app(argc, argv);
-        setupAppInfo(&app);
-
-        KSharedConfig::Ptr config = KSharedConfig::openConfig();
-        KConfigGroup cg(config, "General");
-        checkStamps = cg.readEntry("CheckFileStamps", s_checkStampsDefault);
-
-        runBuildSycoca();
+        KSycoca::self()->ensureCacheValid();
         runKonfUpdate();
         return 0;
     }
 
-    QApplication app(argc, argv);
-    setupAppInfo(&app);
-    app.setApplicationDisplayName("KDE Daemon");
-    app.setQuitOnLastWindowClosed(false);
-
     KDBusService service(KDBusService::Unique);
 
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
@@ -803,7 +716,6 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
 
     bCheckSycoca = cg.readEntry("CheckSycoca", true);
     bCheckUpdates = cg.readEntry("CheckUpdates", true);
-    checkStamps = cg.readEntry("CheckFileStamps", s_checkStampsDefault);
     delayedCheck = cg.readEntry("DelayedCheck", false);
 
 #ifndef Q_OS_WIN
